// cyewindow.cpp
// the main window
// v0.84 (7.3.2002)
// (c) 2002 Andreas Krueger
//
// Please, if you change anything in this code,
// mark the end of those lines with '// *+* your name *+* ' 
// (so that I can find it) and send the code 
// to cye-code@AndreasKrueger.de
//
// last change: 
//          by:

#include <qapplication.h>
#include <qvbuttongroup.h>
#include <qdatetime.h>
#include <qtextstream.h>
#include <qstring.h>

#include "constants.h"
#include "cyewindow.h"
#include "slideshow.h"
#include "counttable.h"
#include "storage.h"
#include "usage.h"
#include "dialogs.h"
#include "client.h"
#include "helpers.h"
#include "exceptions.h"
#include "invitation.h"

class CountTable;

CYEWindow::CYEWindow( QWidget* parent, const char* name, WFlags f )
	: QMainWindow( parent, name, f )
{
	setCaption(tr("Count Your Euro - CYE for Research (v%1 - %1)").arg(PROGRAMVERSION/100.0).arg(__DATE__));

	QHBox *box = new QHBox(this);
	setCentralWidget(box);

	ss=new SlideShow(box);

	QVBox *buttonbox = new QVBox(box);
	QPushButton *buttonInsert=new QPushButton("->\n&Insert",buttonbox,"buttonInsert");
	QPushButton *buttonTakeAway=new QPushButton("<-\nTake Away",buttonbox,"buttonTakeAway");
	buttonInsert->setSizePolicy(QSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding ));
	buttonTakeAway->setSizePolicy(QSizePolicy( QSizePolicy::MinimumExpanding , QSizePolicy::MinimumExpanding ));

	ct=new CountTable(NumCountries,NumCoins,box);

	connect( buttonInsert, SIGNAL(clicked() ), 
		this, SLOT(increaseCurrent()) );
	connect( buttonTakeAway, SIGNAL(clicked() ), 
		this, SLOT(decreaseCurrent()) );

	connect( buttonInsert, SIGNAL(clicked() ), 
		this, SLOT(buttonInsertPressed()) );
	connect( buttonTakeAway, SIGNAL(clicked() ), 
		this, SLOT(buttonTakeAwayPressed()) );

	connect( ct, SIGNAL(pleaseChangeToCoinOfCell(uint,uint)), 
		this, SLOT( changeToCoinOfCell(uint,uint) ) );

	connect (ss, SIGNAL(focusChanged(uint, uint)), 
		this, SLOT( changeFocus(uint,uint)) );

	readyGroupBox=new QVButtonGroup(box);

	QPushButton *buttonReadyAndQuit=new QPushButton("Save purse and QUIT", readyGroupBox,"button Ready and Quit");
	labelStoredPurses= new QLabel(readyGroupBox);
	QPushButton *buttonReadyAndUpload=new QPushButton("Upload purses",readyGroupBox,"button Ready and Upload");
	QPushButton *buttonSettings;
         buttonSettings=new QPushButton("Your settings",readyGroupBox,"button settings");
	QPushButton *buttonGoWebsite;
	     buttonGoWebsite=new QPushButton("See the newest\ndata analysis",readyGroupBox,"button go to WebSite");
	QPushButton *buttonAbout;
         buttonAbout=new QPushButton("about this project",readyGroupBox,"button about project");
	QPushButton *buttonInvite;
    buttonInvite=new QPushButton("invite friends",readyGroupBox,"button invite friends");
	QPushButton *buttonJustQuit; 
         buttonJustQuit=new QPushButton("just QUIT",readyGroupBox,"button resetquit");
	buttonReadyAndQuit->setSizePolicy(QSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding ));
	buttonReadyAndUpload->setSizePolicy(QSizePolicy( QSizePolicy::MinimumExpanding , QSizePolicy::MinimumExpanding ));
	buttonSettings->setSizePolicy(QSizePolicy( QSizePolicy::MinimumExpanding , QSizePolicy::MinimumExpanding ));

	connect (readyGroupBox, SIGNAL(clicked(int)), 
		this, SLOT( readyPressed(int)) );

	// very important after each creation of a CYEWindow: 
	// initializeEverything() !!!
}

void CYEWindow::closeEvent ( QCloseEvent * e ){  // ATTENTION: This is ONLY called when
	quitreason =  CYE_USER_KILLS_PROGRAM_WITH_X; // prg termination use qAppl::exit() !!!
	emit Event("ProgramTerminatedBySysbarOrAltF4");
	Quit();
	e->accept();
}

void CYEWindow::initializeEverything(){
	theData=new ClientStorage();
	int ok=0; 
	try {
		ok=theData->loadFromDisk(FILENAME);
	}
	catch (myException e){
		this->setEnabled(false);
		this->show();
		switch(QMessageBox::warning(this, "file error", errorMessageAndSolution(e.errnum),
			"Ignore\nsaved data.", 
			"Quit now", 0, 0, 1 ) )
		{	case 0: 
				emit Event("BadDatafileOverwrite");
				theData->setDefaultValues();
		  	  break;
			default:
				throw;
			  break;
		}
	}

	switch (ok){
		case 0:
			if (debugging) 	QMessageBox::information( this, "CYE: Settings loaded", "The settings were loaded without problems." );
 		  break;
		default:
			if (debugging) 	QMessageBox::information( this, "CYE: Settings NOT loaded", "No settings file." );
			theData->setDefaultValues();
		  break;
	}
	this->resize( theData->WSize );    // restore size
	this->move( theData->WPosition );  // restore position

	this->setEnabled(false);
	this->show();

	bool dontShowLoadedPicsResults = ( 32==(theData->SettingsFlags & 32) );
	ss->loadPictures(PICFILEPATH, PICFILESUFFIX, dontShowLoadedPicsResults);
	if (dontShowLoadedPicsResults) theData->SettingsFlags |= 32;
	else theData->SettingsFlags &= (65535-32);
	
	theData->NumberOfStartsSinceLastUpload ++;

	theDialogs = new Dialogs(theData, this, "dialogs");

	theInvitations=new Invitations(this,"invitation");
	theDialogs->setPicsAlsoInInvitations(ss->pics, ss->max_size);

	// testing:
	if (debugging)
		theDialogs->TableOfErrorsAndRecommendations(this);

	QString str;
	QTextOStream(&str)<<"Stored Purses on disk: "<<theData->NumFullPurses;
	labelStoredPurses->setText(str);
	labelStoredPurses->setAlignment( AlignHCenter | AlignVCenter );
    labelStoredPurses->setFrameStyle( QFrame::Panel | QFrame::Sunken );

	// before Quitting, the program waits 500 msec to have
	// the event loop finished, then tries to save the data, then quits.
	connect(theDialogs, SIGNAL(QuitEvent(int)), this, SLOT(ProcessQuitEvent(int)) );
	connect(this, SIGNAL(QuitEvent(int)), this, SLOT(ProcessQuitEvent(int)) );
	quitTimer=new QTimer (this, "quitTimer");
	connect(quitTimer, SIGNAL(timeout()), this, SLOT(Quit()) );

	// init the user observation
	Observer = new Usage(this,"usage observer");
	Observer->startObservation(theData->TotalEvents);
	if (debugging){
		theDialogs->createUserObservationWindow(*Observer);
	}
	connect(ss, SIGNAL(Event(QString)), this, SLOT(EventOccurred(QString)) );
	connect(ct, SIGNAL(Event(QString)), this, SLOT(EventOccurred(QString)) );
	connect(this, SIGNAL(Event(QString)), this, SLOT(EventOccurred(QString)) );
	connect(theDialogs, SIGNAL(Event(QString)), this, SLOT(EventOccurred(QString)) );
	// switch ON or OFF the user observation, depending on 2
	// or (just for the first time): switched ON if not yet seen settings 
	// not to loose data of those who will allow it anyway!
	bool allowed= (theData->SettingsFlags & 128) || !(theData->SettingsFlags & 16);
	Observer->toggleObservationAllowed(allowed);

	theData->Sessions.append(*(new Session)); // This session is created as the last in the list
	this->setEnabled(true);
}


void CYEWindow::storePurseAndWindowPosAndUsageInto(ClientStorage *theData, 
									   CountTable *ct, Usage *Observer){
	if (ct->sum!=0) {	// if not empty store this purse
		theData->NumFullPurses ++;
		theData->Sessions.last().SetPurseFromCounttableArray( ct->toVectorArray() ); 
	}
	theData->Sessions.last().SessionEvents=Observer->Session;  // store the user events of this session
	theData->TotalEvents = Observer->TotalEvents;		// store the total sum of user events
	theData->WPosition = this->pos();       // store position
	theData->WSize	   = this->size();       // store size
}

void CYEWindow::ProcessQuitEvent(int reason){
	if (quitTimer->isActive())
		quitTimer->stop();
	quitreason=reason;
	quitTimer->start(500, TRUE);
}

void CYEWindow::Quit(){
	storePurseAndWindowPosAndUsageInto(theData, ct, Observer);

	int ok=theData->saveToDisk(FILENAME);

	this->hide();
	if (ok!=0 && quitreason!=CYE_USER_KILLS_PROGRAM_WITH_X ) {
		switch( QMessageBox::warning( this, "CYE cannot save your data",
                    ERRORSAVING.arg(ok).arg(FILENAME).arg(FILENAME),
					"OK, I try my best.\nPlease return to the program", 
					"No, quit anyway and\ndiscard my data of this session", 0, 0, 1 ) )
		{	case 0: 
				emit Event("WriteErrorDialogReturn");
				this->show();  break;
			default: 
				emit Event("WriteErrorDialogDiscard");
				emit Event("ProgramTerminated");
				QApplication::exit(ok);
				break;
		}
	}
	else {
		if (quitreason ==  USER_DOESNT_WANT_TO_INPUT_NECESSARY_DATA
			|| quitreason ==  CYE_USER_KILLS_PROGRAM_WITH_X){
			QApplication::exit(quitreason);
		}
		else{
			theDialogs->goodbyeShow(ss);
			connect(theDialogs, SIGNAL(ByeByeEnded() ), this, SLOT(quitquit()) );
		}
		emit Event("ProgramTerminated");
	}
}

void CYEWindow::quitquit(){
	QApplication::exit(quitreason);
}

void CYEWindow::EventOccurred(QString eventname){
	Observer->StoreEvent(eventname);
	 //this is only temporary:
	 if (debugging){
		theDialogs->updateEventsViewerWindow(*Observer);
	 }
}
void CYEWindow::buttonInsertPressed(){
	emit Event("Insert");
}
void CYEWindow::buttonTakeAwayPressed(){
	emit Event("TakeAway");
}

void CYEWindow::increaseCurrent(){
	ct->increaseCell(ss->country -1 , ss->coin);
}

void CYEWindow::decreaseCurrent(){
	ct->decreaseCell(ss->country -1, ss->coin);
}

void CYEWindow::changeToCoinOfCell(uint row, uint column) {
	ss->switchToCoin(column, row+1);
}

void CYEWindow::changeFocus(uint coin, uint country){
	ct->setFocusToCell(country-1, coin);
}

void CYEWindow::readyPressed(int button){
	QString temp, text, labeltext;
	switch (button){
	case 0:
		emit Event("SaveQuit");
		emit QuitEvent(0);
	 break;
	case 1:
		emit Event("Upload");
		try{
			if (!theData->NecessarySettingsOK()){
				emit Event("Settings");
				bool firstTime=!(theData->SettingsFlags & 16);
				theDialogs->startSettingsDialog(true, firstTime);
				theData->SettingsFlags |= 16; // Has seen settings
			}
			storePurseAndWindowPosAndUsageInto(theData, ct, Observer);
			if (theData->NecessarySettingsOK()){
				bool sent=false;
				sent=theDialogs->openClientForUploadAndIfSuccessStoreEverything();
				if (sent){
					QString labeltext;
					QTextOStream(&labeltext)<<"Stored Purses on disk: "<<theData->NumFullPurses;
					labelStoredPurses->setText(labeltext);
					ct->reset();
					readyGroupBox->find(button)->setEnabled(false);
					quitreason=CYE_UPLOAD_SUCCESSFUL;
					theDialogs->goodbyeShow(ss);
					connect(theDialogs, SIGNAL(ByeByeEnded() ), this, SLOT(quitquit()) );
				}
			}
		}
		catch (NetErr) { 
			// do nothing? 
			theDialogs->Clientwindow->hide();
		}
		catch (...) {
			throw;  // in a more fatal case....
		};
	   break;
	case 2:
		emit Event("Settings");
		theDialogs->startSettingsDialog(false, false);
		theData->SettingsFlags |= 16; // Has seen settings
		Observer->toggleObservationAllowed(theData->SettingsFlags & 128);
		break;
	case 3:
		emit Event("SeeDataAnalysis");
		
		switch(QMessageBox::information( this, "See data analysis", 
			"Results Downloader not implemented yet. Sorry, coming soon.\n\n"
			"But watch results online: www.AndreasKrueger.de/euro/results",
			"OK, try to open the page now", "Cancel, I remember the URL", 0,0,1))
		{
		case 1:
		   break;
		case 0:
		    emit Event("BrowseResults");
			openBrowser("http://www.AndreasKrueger.de/euro/results");
		   break;
		default:
		   break;
		}
		emit Event("SeeDataAnalysisClosed");
		break;
	case 4:
		emit Event("About");
		switch( theDialogs->About->exec() ) {
			case QMessageBox::Yes:
				// save and exit
				break;
			case QMessageBox::No:
			   emit Event("BrowseHomepage");
			   openBrowser("http://www.AndreasKrueger.de/euro");
			  break;
			case QMessageBox::Cancel:
 			   emit Event("BrowseDownload");
			   openBrowser("http://www.AndreasKrueger.de/euro/download");
			  break;
		}
		emit Event("AboutDialogClosed");
		break;
	case 5:
		emit Event("Invitations");
		theDialogs->CopyFromThenExecInvitationWindowAndCopyTo(theData->InvitationList);
		emit Event("InvitationsClosed");
		break;
	case 6:
		emit Event("Quit");
		if (ct->sum!=0){
			switch( QMessageBox::warning( this, "CYE loosing your data",
                        "If you quit now, this table will be lost!\n\n"
						"Rather press 'SAVE' or 'UPLOAD', if you want to have it stored.",
						"OK, I will not quit", "Quit anyway", 0, 0, 1 ) )
			{	case 0: 
					emit Event("DontQuitAndLooseDataButReturn");
				 break;
				case 1: 
					emit Event("ProgramTerminated");
					ct->reset();
					emit QuitEvent(CYE_USER_DISCARDS_COUNTTABLE_AND_QUITS);
				break;
			}
		}
		else {
			emit Event("ProgramTerminated");
			emit QuitEvent(CYE_USER_DISCARDS_COUNTTABLE_AND_QUITS);
		}
	    break;
	default:
		qDebug("no known button...in CYEWindow::readyPressed(int button)");
	}
	readyGroupBox->find(button)->setDown(false);
}
